-- Advanced Realistic Grass Drying System for Farming Simulator 25
GrassToHay = {}

function GrassToHay:loadMap()
    if self.initialized then return end -- Prevent multiple initializations
    self.cutGrassFields = {}
    self.lastUpdate = 0
    self.initialized = true
    addModEventListener(self)
    print("[GrassToHay] Mod Initialized!")
end

function GrassToHay:update(dt)
    if not g_currentMission or not g_currentMission.environment then return end
    self.lastUpdate = self.lastUpdate + dt
    if self.lastUpdate >= 60000 then -- Check every in-game minute
        self:minuteChanged()
        self.lastUpdate = 0
    end
end

function GrassToHay:minuteChanged()
    if not g_currentMission or not g_currentMission.environment then return end
    local currentTime = g_currentMission.environment.dayTime
    local weatherSystem = g_currentMission.environment.weather
    if not weatherSystem then return end
    
    local isRaining = weatherSystem:getIsRaining() or false
    local temperature = weatherSystem.getCurrentTemperature and weatherSystem:getCurrentTemperature() or 20

    for fieldId, data in pairs(self.cutGrassFields) do
        local timeElapsed = currentTime - data.cutTime
        local dryingMultiplier = 1.0 

        -- Adjust drying speed based on weather conditions
        if isRaining then
            dryingMultiplier = 0  -- No drying when raining
            print("[GrassToHay] Drying stopped due to rain on field " .. fieldId)
        elseif temperature > 25 then
            dryingMultiplier = 1.5  -- Faster drying on hot days
        end

        local adjustedElapsedTime = timeElapsed * dryingMultiplier

        if adjustedElapsedTime >= 43200000 and not data.stage1 then -- 12 in-game hours
            self:updateGrassStage(fieldId, 1)
            self.cutGrassFields[fieldId].stage1 = true
        elseif adjustedElapsedTime >= 86400000 and data.stage1 then -- 24 in-game hours
            self:convertToHay(fieldId)
        end
    end
end

function GrassToHay:updateGrassStage(fieldId, stage)
    local field = g_fieldManager:getFieldById(fieldId)
    if field and field.foliageLayers then
        local grassLayer = field.foliageLayers.grass
        if grassLayer then
            setDensityParallelogram(grassLayer, field.startX, field.startZ, field.widthX, field.widthZ, 0, 1, stage)
            print("[GrassToHay] Field " .. fieldId .. " is partially dried (Stage " .. stage .. ").")
        else
            print("[GrassToHay] Error: No valid grass layer found for field " .. fieldId)
        end
    else
        print("[GrassToHay] Error: Invalid field data for " .. fieldId)
    end
end

function GrassToHay:convertToHay(fieldId)
    local field = g_fieldManager:getFieldById(fieldId)
    if field and field.foliageLayers then
        local grassLayer = field.foliageLayers.grass
        local hayLayer = field.foliageLayers.hay or field.foliageLayers.windrow
        if grassLayer and hayLayer then
            setDensityParallelogram(grassLayer, field.startX, field.startZ, field.widthX, field.widthZ, 0, 1, 0) -- Remove grass
            setDensityParallelogram(hayLayer, field.startX, field.startZ, field.widthX, field.widthZ, 0, 1, 1) -- Set hay
            print("[GrassToHay] Field " .. fieldId .. " fully converted to hay!")
            self.cutGrassFields[fieldId] = nil
        else
            print("[GrassToHay] Error: Missing foliage layers for field " .. fieldId)
        end
    else
        print("[GrassToHay] Error: Invalid field data for " .. fieldId)
    end
end

function GrassToHay:cutGrass(fieldId)
    local field = g_fieldManager:getFieldById(fieldId)
    if field then
        self.cutGrassFields[fieldId] = { cutTime = g_currentMission.environment.dayTime, stage1 = false }
        print("[GrassToHay] Field " .. fieldId .. " registered for drying!")
    else
        print("[GrassToHay] Error: Invalid field ID " .. fieldId)
    end
end

addModEventListener(GrassToHay)
